package com.huawei.appmate_flutter;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import androidx.annotation.NonNull;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.huawei.appmate.PurchaseClient;
import com.huawei.appmate.callback.PurchaseResultListener;
import com.huawei.appmate.callback.ReceivedDataListener;
import com.huawei.appmate.data.remote.response.UserRelation;
import com.huawei.appmate.domain.model.PlatformServiceType;
import com.huawei.appmate.model.Product;
import com.huawei.appmate.model.ProductType;
import com.huawei.appmate.model.PurchaseInfo;
import com.huawei.appmate.model.PurchaseRequest;
import com.huawei.appmate.model.PurchaseResultInfo;
import com.huawei.appmate.model.GenericError;

import org.jetbrains.annotations.NotNull;

import static com.huawei.appmate.model.ConstantKt.REQ_CODE_BUY;

import java.util.ArrayList;
import java.util.List;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;

/** AppmateFlutterPlugin */
public class AppmateFlutterPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware, PluginRegistry.ActivityResultListener {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private static Application application;
  private static Activity activity;
  private Gson gson = new Gson();

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    Context context = flutterPluginBinding.getApplicationContext();
    if (context instanceof Application) {
      application = (Application) context;
    }
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "appmate_flutter");
    channel.setMethodCallHandler(this);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("setApiKey")) {
      setApiKey(call, result);
    } else if (call.method.equals("setSandboxActive")) {
      setSandboxActive(call, result);
    } else if (call.method.equals("setUserId")) {
      setUserId(call, result);
    } else if (call.method.equals("getUserId")) {
      getUserId(call, result);
    } else if (call.method.equals("getProducts")) {
      getProducts(call, result);
    } else if (call.method.equals("getProductsWithType")) {
      getProductsWithType(call, result);
    } else if (call.method.equals("getProductsWithIdList")) {
      getProductsWithIdList(call, result);
    } else if (call.method.equals("getCurrentPurchases")) {
      getCurrentPurchases(call, result);
    } else if (call.method.equals("getAllPurchases")) {
      getAllPurchases(call, result);
    } else if (call.method.equals("makePurchase")) {
      makePurchase(call, result);
    } else if (call.method.equals("restorePurchases")) {
      restorePurchases(call, result);
    } else if (call.method.equals("consumePurchase")) {
      consumePurchase(call, result);
    } else if (call.method.equals("consumePurchases")) {
      consumePurchases(call, result);
    } else if (call.method.equals("unsubscribe")) {
      unsubscribe(call, result);
    } else if (call.method.equals("createUserIdRelation")) {
      createUserIdRelation(call, result);
    } else if (call.method.equals("deleteMasterUserIdRelation")) {
      deleteMasterUserIdRelation(call, result);
    } else if (call.method.equals("deleteSubUserIdRelation")) {
      deleteSubUserIdRelation(call, result);
    } else if (call.method.equals("setPlatformType")) {
      setPlatformType(call, result);
    } else if (call.method.equals("getPlatformType")) {
      getPlatformType(call, result);
    } else if (call.method.equals("setRuntimeEnvironment")) {
      setRuntimeEnvironment(call, result);
    } else {
      result.notImplemented();
    }
  }

  private void sendResult(Result result, Object data, String error) {
    List<Object> response = new ArrayList<>();
    response.add(data);
    response.add(error != null ? new com.huawei.appmate_flutter.GenericError(error).toString() : null);
    result.success(response);
  }

  private void setApiKey(@NonNull MethodCall call, @NonNull Result result) {
    String apiKey = call.argument("apiKey");
    if (apiKey == null) {
      sendResult(result, false, "bad arguments");
    } else if (application == null) {
      sendResult(result, false, "set plugin application");
    } else {
      PurchaseClient.getInstance(application, apiKey);
      sendResult(result, true, null);
    }
  }

  private void setSandboxActive(@NonNull MethodCall call, @NonNull Result result) {
    sendResult(result, true, null);
  }

  private void setUserId(@NonNull MethodCall call, @NonNull Result result) {
    String userId = call.argument("userId");
    if (userId == null) {
      sendResult(result, false, "bad arguments");
    } else {
      PurchaseClient.getInstance().setUserId(userId);
      sendResult(result, true, null);
    }
  }

  private void getUserId(@NonNull MethodCall call, @NonNull final Result result) {
    PurchaseClient.getInstance().getUserId(new ReceivedDataListener<String, GenericError>() {
      @Override
      public void onSucceeded(String s) {
        result.success(s);
      }

      @Override
      public void onError(GenericError error) {
        result.success("");
      }
    });
  }

  private void getProducts(@NonNull MethodCall call, @NonNull final Result result) {
    PurchaseClient.getInstance().getProducts(new ReceivedDataListener<List<Product>, GenericError>() {
      @Override
      public void onSucceeded(List<Product> products) {
        String json = gson.toJson(products);
        sendResult(result, json, null);
      }

      @Override
      public void onError(GenericError genericError) {
        sendResult(result, null, genericError.getErrorMessage());
      }
    });
  }

  private void getProductsWithType(@NonNull MethodCall call, @NonNull final Result result) {
    String type = call.argument("type");
    if (type == null) {
      sendResult(result, false, "bad arguments");
    } else {
      ProductType productType = ProductType.SUBSCRIPTION;
      if (type.contentEquals("0")) {
        productType = ProductType.CONSUMABLE;
      } else if (type.contentEquals("1")) {
        productType = ProductType.NONCONSUMABLE;
      }
      PurchaseClient.getInstance().getProductsByProductType(productType, new ReceivedDataListener<List<Product>, GenericError>() {
        @Override
        public void onSucceeded(List<Product> products) {
          String json = gson.toJson(products);
          sendResult(result, json, null);
        }

        @Override
        public void onError(GenericError genericError) {
          sendResult(result, null, genericError.getErrorMessage());
        }
      });
    }
  }

  private void getProductsWithIdList(@NonNull MethodCall call, @NonNull final Result result) {
    List<String> ids = call.argument("ids");
    if (ids == null || ids.size() == 0) {
      sendResult(result, false, "bad arguments");
    } else {
      PurchaseClient.getInstance().getProductsByProductIdList(ids, new ReceivedDataListener<List<Product>, GenericError>() {
        @Override
        public void onSucceeded(List<Product> products) {
          String json = gson.toJson(products);
          sendResult(result, json, null);
        }

        @Override
        public void onError(GenericError genericError) {
          sendResult(result, null, genericError.getErrorMessage());
        }
      });
    }
  }

  private void getCurrentPurchases(@NonNull MethodCall call, @NonNull final Result result) {
    PurchaseClient.getInstance().getPurchases(new ReceivedDataListener<List<PurchaseInfo>, GenericError>() {
      @Override
      public void onSucceeded(List<PurchaseInfo> purchaseInfos) {
        String json = gson.toJson(purchaseInfos);
        sendResult(result, json, null);
      }

      @Override
      public void onError(GenericError genericError) {
        sendResult(result, null, genericError.getErrorMessage());
      }
    });
  }

  private void getAllPurchases(@NonNull MethodCall call, @NonNull final Result result) {
    PurchaseClient.getInstance().getPurchasesHistory(new ReceivedDataListener<List<PurchaseInfo>, GenericError>() {
      @Override
      public void onSucceeded(List<PurchaseInfo> purchaseInfos) {
        String json = gson.toJson(purchaseInfos);
        sendResult(result, json, null);
      }

      @Override
      public void onError(GenericError genericError) {
        sendResult(result, null, genericError.getErrorMessage());
      }
    });
  }

  private void makePurchase(@NonNull MethodCall call, @NonNull final Result result) {
    String strProduct = call.argument("product");
    PurchaseRequest purchaseRequest = null;
    JsonObject jsonProduct;
    if (strProduct != null) {
      try {

        jsonProduct = new JsonParser().parse(strProduct).getAsJsonObject();

        String id = jsonProduct.get("productId").getAsString();
        String type = jsonProduct.get("purchaseType").getAsString();
        ProductType productType = ProductType.SUBSCRIPTION;
        if (type.contentEquals("0")) {
          productType = ProductType.CONSUMABLE;
        } else if (type.contentEquals("1")) {
          productType = ProductType.NONCONSUMABLE;
        }

        purchaseRequest = new PurchaseRequest(id, productType);
      } catch (Exception e) { e.printStackTrace(); }
    }

    if (purchaseRequest == null) {
      sendResult(result, false, "bad arguments");
    } else if (activity == null) {
      sendResult(result, false, "set plugin activity");
    } else {
      PurchaseClient.getInstance().purchase(activity,
              purchaseRequest,
              new PurchaseResultListener<PurchaseResultInfo, GenericError>() {
                @Override
                public void onSuccess(PurchaseResultInfo purchaseResultInfo) {
                  String json = gson.toJson(purchaseResultInfo);
                  sendResult(result, json, null);
                }

                @Override
                public void onError(GenericError genericError) {
                  sendResult(result, null, genericError.getErrorMessage());
                }
              });
    }
  }

  private void restorePurchases(@NonNull MethodCall call, @NonNull Result result) {
    List<PurchaseInfo> purchaseInfos = new ArrayList<>();
    String json = gson.toJson(purchaseInfos);
    sendResult(result, json, null);
  }

  private void consumePurchase(@NonNull MethodCall call, @NonNull final Result result) {
    String purchaseToken = call.argument("purchaseToken");
    if (purchaseToken == null) {
      sendResult(result, false, "bad arguments");
    } else {
      PurchaseClient.getInstance().consumePurchase(purchaseToken, new ReceivedDataListener<String, GenericError>() {
        @Override
        public void onSucceeded(String s) {
          sendResult(result, true, null);
        }

        @Override
        public void onError(GenericError genericError) {
          sendResult(result, false, genericError.getErrorMessage());
        }
      });
    }
  }

  private void consumePurchases(@NonNull MethodCall call, @NonNull final Result result) {
    List<String> purchaseTokens = call.argument("purchaseTokens");
    if (purchaseTokens == null || purchaseTokens.size() == 0) {
      sendResult(result, false, "bad arguments");
    } else {
      consumePurchases(result, purchaseTokens, 0, null);
    }
  }

  private void consumePurchases(@NonNull final Result result, final List<String> purchaseTokens, final int index, final String error) {
    if (index < purchaseTokens.size()) {
      PurchaseClient.getInstance().consumePurchase(purchaseTokens.get(index), new ReceivedDataListener<String, GenericError>() {
        @Override
        public void onSucceeded(String s) {
          consumePurchases(result, purchaseTokens, index+1, error);
        }

        @Override
        public void onError(GenericError genericError) {
          consumePurchases(result, purchaseTokens, index+1, genericError.getErrorMessage());
        }
      });
    } else {
      sendResult(result, error == null, error);
    }
  }

  private void unsubscribe(@NonNull MethodCall call, @NonNull final Result result) {
    String purchaseToken = call.argument("purchaseToken");
    if (purchaseToken == null) {
      sendResult(result, false, "bad arguments");
    } else {
      PurchaseClient.getInstance().unsubscribe(purchaseToken, new ReceivedDataListener<Boolean, GenericError>() {
        @Override
        public void onSucceeded(Boolean response) {
          sendResult(result, response, null);
        }

        @Override
        public void onError(GenericError error) {
          sendResult(result, false, error.getErrorMessage());
        }
      });
    }
  }

  private void createUserIdRelation(@NonNull MethodCall call, @NonNull final Result result) {
    String masterUserId = call.argument("masterUserId");
    if (masterUserId == null) {
      sendResult(result, false, "bad arguments");
    } else {
      PurchaseClient.getInstance().createUserIdRelation(masterUserId, new ReceivedDataListener<UserRelation, GenericError>() {
        @Override
        public void onSucceeded(UserRelation response) {
          String json = gson.toJson(response);
          sendResult(result, json, null);
        }

        @Override
        public void onError(GenericError error) {
          sendResult(result, null, error.getErrorMessage());
        }
      });
    }
  }

  private void deleteMasterUserIdRelation(@NonNull MethodCall call, @NonNull final Result result) {
    String masterUserId = call.argument("masterUserId");
    if (masterUserId == null) {
      sendResult(result, false, "bad arguments");
    } else {
      PurchaseClient.getInstance().deleteMasterUserIdRelation(masterUserId, new ReceivedDataListener<Boolean, GenericError>() {
        @Override
        public void onSucceeded(Boolean response) {
          sendResult(result, response, null);
        }

        @Override
        public void onError(GenericError error) {
          sendResult(result, false, error.getErrorMessage());
        }
      });
    }
  }

  private void deleteSubUserIdRelation(@NonNull MethodCall call, @NonNull final Result result) {
    String subUserId = call.argument("subUserId");
    if (subUserId == null) {
      sendResult(result, false, "bad arguments");
    } else {
      PurchaseClient.getInstance().deleteSubUserIdRelation(subUserId, new ReceivedDataListener<Boolean, GenericError>() {
        @Override
        public void onSucceeded(Boolean response) {
          sendResult(result, response, null);
        }

        @Override
        public void onError(GenericError error) {
          sendResult(result, false, error.getErrorMessage());
        }
      });
    }
  }

  private void setPlatformType(@NonNull MethodCall call, @NonNull final Result result) {
    String platformType = call.argument("platformType");
    if (platformType == null || (!platformType.contentEquals("HMS") && !platformType.contentEquals("GMS"))) {
      sendResult(result, false, "bad arguments");
    } else {
      PlatformServiceType platformServiceType;
      if (platformType.contentEquals("HMS")) {
        platformServiceType = PlatformServiceType.HMS;
      } else {
        platformServiceType = PlatformServiceType.GMS;
      }
      PurchaseClient.getInstance().setPlatformType(platformServiceType, new ReceivedDataListener<Boolean, GenericError>() {
        @Override
        public void onSucceeded(Boolean response) {
          PurchaseClient.getInstance().sharedPreference.a("KEY_QUERY_PRODUCT_TIME", 0);
          PurchaseClient.getInstance().sharedPreference.a("KEY_QUERY_PURCHASE_TIME", 0);
          sendResult(result, response, null);
        }

        @Override
        public void onError(GenericError error) {
          sendResult(result, false, error.getErrorMessage());
        }
      });
    }
  }

  private void getPlatformType(@NonNull MethodCall call, @NonNull final Result result) {
    String platformType = "";
    switch (PurchaseClient.getInstance().getPlatformType()) {
      case HMS:
        platformType = "HMS";
        break;
      case GMS:
        platformType = "GMS";
        break;
      case UNSUPPORTED:
        platformType = "";
        break;
    }
    result.success(platformType);
  }

  private void setRuntimeEnvironment(@NonNull MethodCall call, @NonNull final Result result) {
    String isTestStr = call.argument("environment");
    if (isTestStr == null || (!isTestStr.contentEquals("test") && !isTestStr.contentEquals("prod") && !isTestStr.contentEquals("preprod"))) {
      sendResult(result, false, "bad arguments");
    } else {
      if (isTestStr.contentEquals("test")) {
        PurchaseClient.getInstance().sharedPreference.a("ServiceUrlType", "usp_test_service");
      } else if (isTestStr.contentEquals("preprod")) {
        PurchaseClient.getInstance().sharedPreference.a("ServiceUrlType", "usp_pre_prod_service");
      } else {
        PurchaseClient.getInstance().sharedPreference.a("ServiceUrlType", "usp_prod_service");
      }

      PurchaseClient.getInstance().sharedPreference.a("KEY_QUERY_PRODUCT_TIME", 0);
      PurchaseClient.getInstance().sharedPreference.a("KEY_QUERY_PURCHASE_TIME", 0);

      String packageName = activity.getPackageName();
      PackageManager pm = activity.getPackageManager();
      Intent intent = pm.getLaunchIntentForPackage(packageName);
      intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
      intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK);
      activity.startActivity(intent);
      Runtime.getRuntime().exit(0);
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  @Override
  public void onAttachedToActivity(@NonNull @NotNull ActivityPluginBinding binding) {
    activity = binding.getActivity();
    binding.addActivityResultListener(this);
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull @NotNull ActivityPluginBinding binding) {

  }

  @Override
  public void onDetachedFromActivity() {

  }

  @Override
  public boolean onActivityResult(int requestCode, int resultCode, Intent data) {
    if (requestCode == REQ_CODE_BUY) {
      PurchaseClient.getInstance().getResultListener().onActivityResult(data);
    }
    return false;
  }
}
