import 'dart:developer';

import 'package:appmate_flutter_example/util.dart';
import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:appmate_flutter/PurchaseClient.dart';
import 'package:appmate_flutter/models.dart';
import 'package:flutter/cupertino.dart';
import 'package:shared_preferences/shared_preferences.dart';

class Settings extends StatefulWidget {
  const Settings({Key? key}) : super(key: key);

  @override
  _SettingsState createState() => _SettingsState();
}

class _SettingsState extends State<Settings> {

String _platformType = "";

  @override
  void initState() {
    super.initState();
    getPlatformType();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      resizeToAvoidBottomInset: false,
      backgroundColor: Colors.grey[200],
      appBar: AppBar(
        title: Text('AppMate Example'),
        centerTitle: false,
        backgroundColor: Colors.redAccent,
      ),
      body: SingleChildScrollView(child: _buildSettings()),
    );
  }

  Future<void> getPlatformType() async {
    String? response;
    try {
      response = await PurchaseClient.getPlatformType();
    } on PlatformException {}

    if (!mounted) return;

    setState(() {
      _platformType = response ?? "-";
    });
  }

  Future<void> setPlatformType(String platformType) async {
    BaseResponse response;
    try {
      response = await PurchaseClient.setPlatformType(platformType);
      if (response.error != null) {
        Util.showDialog(context, response.error!.message ?? "Unknown Error");
      } else {
        getPlatformType();
        Util.showDialog(context, response.success ? "Set Platform Type Success" : "Unknown Error");
      }
    } on PlatformException {}
  }

  Future<void> setRuntimeEnvironment(String environment) async {
    BaseResponse response;
    try {
      final prefs = await SharedPreferences.getInstance();
      prefs.setString('_environment', environment);
      response = await PurchaseClient.setRuntimeEnvironment(environment);
      if (response.error != null) {
        Util.showDialog(context, response.error!.message ?? "Unknown Error");
      } else {
        Util.showDialog(context, response.success ? "Set Runtime Environment Success" : "Unknown Error");
      }
    } on PlatformException {}
  }

  Widget _buildSettings() {
    return 
      Padding(padding: const EdgeInsets.all(20),
      child: Column(
      children: [
        Padding(
          padding: const EdgeInsets.only(top: 20),
          child: Column(children: [Text("Current Platform Type", textAlign: TextAlign.center), Text(_platformType, textAlign: TextAlign.center)],)
        ),
        Padding(
          padding: const EdgeInsets.only(top: 20),
          child: Row(children: [
                  Expanded(
                    flex: 1,
                    child:  OutlinedButton(onPressed: () { setPlatformType("HMS"); }, child: Text("HMS"))
                  ),
                  Expanded(
                    flex: 1,
                    child: OutlinedButton(onPressed: () { setPlatformType("GMS"); }, child: Text("GMS"))
                  )
                ]),
        ),
        Padding(
          padding: const EdgeInsets.only(top: 50),
          child: Column(children: [Text("Set Runtime Environment", textAlign: TextAlign.center)],)
        ),
        Padding(
          padding: const EdgeInsets.only(top: 20),
          child: Row(children: [
                  Expanded(
                    flex: 1,
                    child:  OutlinedButton(onPressed: () { setRuntimeEnvironment("prod"); }, child: Text("PROD"))
                  ),
                  Expanded(
                    flex: 1,
                    child:  OutlinedButton(onPressed: () { setRuntimeEnvironment("preprod"); }, child: Text("PREPROD"))
                  ),
                  Expanded(
                    flex: 1,
                    child: OutlinedButton(onPressed: () { setRuntimeEnvironment("test"); }, child: Text("TEST"))
                  )
                ]),
        ),
      ],
    ));
  }

}
