import 'dart:developer';

import 'package:appmate_flutter_example/util.dart';
import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:appmate_flutter/PurchaseClient.dart';
import 'package:appmate_flutter/models.dart';
import 'package:flutter/cupertino.dart';

class UserIdRelation extends StatefulWidget {
  const UserIdRelation({Key? key}) : super(key: key);

  @override
  _UserIdRelationState createState() => _UserIdRelationState();
}

class _UserIdRelationState extends State<UserIdRelation> {

  final textController = TextEditingController();
  Object segmentedControlGroupValue = 0;
  String infoText = "";
  String _myUserId = "";

  @override
  void initState() {
    super.initState();
    updateUI();
    getUserId();
  }

  @override
  void dispose() {
    textController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      resizeToAvoidBottomInset: false,
      backgroundColor: Colors.grey[200],
      appBar: AppBar(
        title: Text('AppMate Example'),
        centerTitle: false,
        backgroundColor: Colors.redAccent,
      ),
      body: SingleChildScrollView(child: _buildUserIdRelation()),
    );
  }

  void updateUI() async {
    if (segmentedControlGroupValue == 0) {
      infoText = "Type master user id to relate current user as a sub user.";
    } else if (segmentedControlGroupValue == 1) {
      infoText = "Type sub user id to delete sub user id's master user relation.\nType master user id to delete master user id's sub user relations.";
    }
  }

  Future<void> getUserId() async {
    String? response;
    try {
      response = await PurchaseClient.getUserId();
    } on PlatformException {}

    if (!mounted) return;

    setState(() {
      _myUserId = response ?? "-";
    });
  }

  Future<void> createUserIdRelation() async {
    CreateUserRelationResponse response;
    try {
      response = await PurchaseClient.createUserIdRelation(textController.text);
      if (response.error != null) {
        Util.showDialog(context, response.error!.message ?? "Unknown Error");
      } else {
        Util.showDialog(context, (response.data != null) ? "Create Success" : "Unknown Error");
      }
    } on PlatformException {}
  }

  Future<void> deleteMasterUserIdRelation() async {
    BaseResponse response;
    try {
      response = await PurchaseClient.deleteMasterUserIdRelation(textController.text);
      if (response.error != null) {
        Util.showDialog(context, response.error!.message ?? "Unknown Error");
      } else {
        Util.showDialog(context, response.success ? "Delete Master User Relation Success" : "Unknown Error");
      }
    } on PlatformException {}
  }

  Future<void> deleteSubUserIdRelation() async {
    BaseResponse response;
    try {
      response = await PurchaseClient.deleteSubUserIdRelation(textController.text);
      if (response.error != null) {
        Util.showDialog(context, response.error!.message ?? "Unknown Error");
      } else {
        Util.showDialog(context, response.success ? "Delete Sub User Relation Success" : "Unknown Error");
      }
    } on PlatformException {}
  }

  Widget _buildUserIdRelation() {
    return 
      Padding(padding: const EdgeInsets.all(20),
      child: Column(
      children: [
        CupertinoSlidingSegmentedControl(
              groupValue: segmentedControlGroupValue,
              children: {
                0: Text("Create Relation"),
                1: Text("Delete Relation")
              },
              onValueChanged: (i) {
                setState(() {
                  segmentedControlGroupValue = i!;
                  updateUI();
                });
              }),
        Padding(
          padding: const EdgeInsets.only(top: 20),
          child: Text(infoText,
          textAlign: TextAlign.center)
        ),
        Padding(
          padding: const EdgeInsets.only(top: 20),
          child: TextField(
            controller: textController,
            decoration: const InputDecoration(
              border: OutlineInputBorder(),
              hintText: 'User Id'
            ),
          )
        ),
        Padding(
          padding: const EdgeInsets.only(top: 20),
          child: _getButtons()
        ),
        OutlinedButton(onPressed: () { Clipboard.setData(new ClipboardData(text: _myUserId)); }, child: Text(_myUserId))
      ],
    ));
  }

  Widget _getButtons() {
    if (segmentedControlGroupValue == 0) {
      return OutlinedButton(onPressed: () { createUserIdRelation(); }, child: Text("Create Relation"));
    } else {
      return Row(children: [
        Expanded(
          flex: 1,
          child:  OutlinedButton(onPressed: () { deleteSubUserIdRelation(); }, child: Text("Delete Sub Relation"))
        ),
        Expanded(
          flex: 1,
          child: OutlinedButton(onPressed: () { deleteMasterUserIdRelation(); }, child: Text("Delete Master Relation"))
        )
      ]);
    }
  }

}
