import Flutter
import UIKit
import appmate

public class SwiftAppmateFlutterPlugin: NSObject, FlutterPlugin {
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "appmate_flutter", binaryMessenger: registrar.messenger())
        let instance = SwiftAppmateFlutterPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if call.method == "setApiKey" {
            setApiKey(call, result: result)
        } else if call.method == "setSandboxActive" {
            setSandboxActive(call, result: result)
        } else if call.method == "setUserId" {
            setUserId(call, result: result)
        } else if call.method == "getUserId" {
            getUserId(call, result: result)
        } else if call.method == "getProducts" {
            getProducts(call, result: result)
        } else if call.method == "getProductsWithType" {
            getProductsWithType(call, result: result)
        } else if call.method == "getProductsWithIdList" {
            getProductsWithIdList(call, result: result)
        } else if call.method == "getCurrentPurchases" {
            getCurrentPurchases(call, result: result)
        } else if call.method == "getAllPurchases" {
            getAllPurchases(call, result: result)
        } else if call.method == "makePurchase" {
            makePurchase(call, result: result)
        } else if call.method == "restorePurchases" {
            restorePurchases(call, result: result)
        } else if call.method == "consumePurchase" {
            consumePurchase(call, result: result)
        } else if call.method == "consumePurchases" {
            consumePurchases(call, result: result)
        } else if call.method == "unsubscribe" {
            unsubscribe(call, result: result)
        } else if call.method == "createUserIdRelation" {
            createUserIdRelation(call, result: result)
        } else if call.method == "deleteMasterUserIdRelation" {
            deleteMasterUserIdRelation(call, result: result)
        } else if call.method == "deleteSubUserIdRelation" {
            deleteSubUserIdRelation(call, result: result)
        } else if call.method == "setRuntimeEnvironment" {
            setRuntimeEnvironment(call, result: result)
        }
    }
    
    private func setApiKey(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let apiKey = args["apiKey"] as? String {
            PurchaseClient.shared.setApiKey(apiKey)
            result([true, nil])
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func setSandboxActive(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let isActive = args["isActive"] as? Bool {
            PurchaseClient.shared.setSandboxActive(isActive)
            result([true, nil])
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func setUserId(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let userId = args["userId"] as? String {
            PurchaseClient.shared.setUserId(userId)
            result([true, nil])
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func getUserId(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        result(PurchaseClient.shared.getUserId())
    }
    
    private func getProducts(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        PurchaseClient.shared.getProducts { products, error in
            if let products = products {
                result([products.description, nil])
            } else {
                result([nil, error?.description])
            }
        }
    }
    
    private func getProductsWithType(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let type = args["type"] as? String {
            PurchaseClient.shared.getProductsWithType(type) { products, error in
                if let products = products {
                    result([products.description, nil])
                } else {
                    result([nil, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func getProductsWithIdList(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let ids = args["ids"] as? [String] {
            PurchaseClient.shared.getProductsWithIdList(ids) { products, error in
                if let products = products {
                    result([products.description, nil])
                } else {
                    result([nil, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func getCurrentPurchases(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        PurchaseClient.shared.getCurrentPurchases { purchases, error in
            if let purchases = purchases {
                result([purchases.description, nil])
            } else {
                result([nil, error?.description])
            }
        }
    }
    
    private func getAllPurchases(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        PurchaseClient.shared.getAllPurchases { purchases, error in
            if let purchases = purchases {
                result([purchases.description, nil])
            } else {
                result([nil, error?.description])
            }
        }
    }
    
    private func makePurchase(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        let decoder = DescriptionDecoder()
        
        if let args = call.arguments as? Dictionary<String, Any>, let strProduct = args["product"] as? String, let product = decoder.decode(Product.self, from: strProduct) {
            PurchaseClient.shared.makePurchase(with: product) { purchaseResult, purchaseCode in
                if purchaseResult != nil {
                    result([purchaseResult!.description, nil])
                } else {
                    let error = GenericError(message: purchaseCode.description)
                    result([nil, error.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func restorePurchases(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        PurchaseClient.shared.restorePurchases { purchases, error in
            if let purchases = purchases {
                result([purchases.description, nil])
            } else {
                result([nil, error?.description])
            }
        }
    }
    
    private func consumePurchase(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let purchaseToken = args["purchaseToken"] as? String {
            PurchaseClient.shared.consumePurchaseWithPurchaseToken(purchaseToken) { response, error in
                if let response = response {
                    result([true, nil])
                } else {
                    result([false, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([false, error.description])
        }
    }
    
    private func consumePurchases(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let purchaseTokens = args["purchaseTokens"] as? [String] {
            PurchaseClient.shared.consumePurchasesWithPurchaseTokens(purchaseTokens) { response, error in
                if let response = response {
                    result([true, nil])
                } else {
                    result([false, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([false, error.description])
        }
    }
    
    private func unsubscribe(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        PurchaseClient.shared.unsubscribe()
        result([true, nil])
    }
    
    private func createUserIdRelation(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let masterUserId = args["masterUserId"] as? String {
            PurchaseClient.shared.createUserIdRelation(with: masterUserId) { response, error in
                if let response = response, let data = response.data {
                    result([data.description, nil])
                } else {
                    result([nil, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func deleteMasterUserIdRelation(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let masterUserId = args["masterUserId"] as? String {
            PurchaseClient.shared.deleteMasterUserIdRelation(for: masterUserId) { response, error in
                if let response = response {
                    result([true, nil])
                } else {
                    result([false, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([false, error.description])
        }
    }
    
    private func deleteSubUserIdRelation(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let subUserId = args["subUserId"] as? String {
            PurchaseClient.shared.deleteSubUserIdRelation(for: subUserId) { response, error in
                if let response = response {
                    result([true, nil])
                } else {
                    result([false, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([false, error.description])
        }
    }
    
    private func setRuntimeEnvironment(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let environment = args["environment"] as? String, (environment == "test" || environment == "prod" || environment == "preprod") {
            PurchaseClient.shared.setRuntimeEnvironment(environment)
            result([true, nil])
        } else {
            let error = GenericError(message: "bad arguments")
            result([false, error.description])
        }
    }
    
}
