import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:flutter/services.dart';

import 'models.dart';

class PurchaseClient {
  static const MethodChannel _channel =
      const MethodChannel('appmate_flutter');

  static Future<BaseResponse> setApiKey(String apiKey) async {
    List<Object?>? response = await _channel.invokeMethod('setApiKey', {"apiKey": apiKey});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error = GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  static Future<BaseResponse> setSandboxActive(bool isActive) async {
    List<Object?>? response = await _channel.invokeMethod('setSandboxActive', {"isActive": isActive});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error = GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  static Future<BaseResponse> setUserId(String userId) async {
    List<Object?>? response = await _channel.invokeMethod('setUserId', {"userId": userId});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error = GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  static Future<String> getUserId() async {
    final String? result = await _channel.invokeMethod('getUserId');
    if (result != null) {
      return result;
    }
    return "";
  }

  static Future<BaseResponse> setPlatformType(String platformType) async {
    if (!Platform.isAndroid) {
      BaseResponse result = new BaseResponse(false, GenericError('Unsupported platform!'));
      return result;
    }
    List<Object?>? response = await _channel.invokeMethod('setPlatformType', {"platformType": platformType});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error = GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  static Future<String> getPlatformType() async {
    if (!Platform.isAndroid) {
      return "";
    }
    String response = await _channel.invokeMethod('getPlatformType');
    return response;
  }

  static Future<BaseResponse> setRuntimeEnvironment(String environment) async {
    List<Object?>? response = await _channel.invokeMethod('setRuntimeEnvironment', {"environment": environment});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error = GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  static Future<ProductsResponse> getProducts() async {
    final List<Object?>? response = await _channel.invokeMethod('getProducts');
    return _getProductsFromResponse(response);
  }

  static Future<ProductsResponse> getProductsWithType(String type) async {
    final List<Object?>? response = await _channel.invokeMethod('getProductsWithType', {"type": type});
    return _getProductsFromResponse(response);
  }

  static Future<ProductsResponse> getProductsWithIdList(List<String> ids) async {
    final List<Object?>? response = await _channel.invokeMethod('getProductsWithIdList', {"ids": ids});
    return _getProductsFromResponse(response);
  }

  static ProductsResponse _getProductsFromResponse(List<Object?>? response) {
    if (response != null) {
      List<Product>? products;
      GenericError? error;
      if (response[0] != null) {
        products = List<Product>.from(json.decode(response[0]!.toString()).map((model) => Product.fromJson(model)));
      }
      if (response[1] != null) {
        error = GenericError.fromJson(json.decode(response[1]!.toString()));
      }
      return new ProductsResponse(products, error);
    }
    return new ProductsResponse(null, null);
  }

  static Future<PurchasesResponse> getCurrentPurchases() async {
    List<Object?>? response = await _channel.invokeMethod('getCurrentPurchases');
    return _getPurchasesFromResponse(response);
  }

  static Future<PurchasesResponse> getAllPurchases() async {
    List<Object?>? response = await _channel.invokeMethod('getAllPurchases');
    return _getPurchasesFromResponse(response);
  }

  static Future<PurchasesResponse> restorePurchases() async {
    List<Object?>? response = await _channel.invokeMethod('restorePurchases');
    return _getPurchasesFromResponse(response);
  }

  static PurchasesResponse _getPurchasesFromResponse(List<Object?>? response) {
    if (response != null) {
      List<PurchaseInfo>? purchases;
      GenericError? error;
      if (response[0] != null) {
        purchases = List<PurchaseInfo>.from(json.decode(response[0]!.toString()).map((model) => PurchaseInfo.fromJson(model)));
      }
      if (response[1] != null) {
        error = GenericError.fromJson(json.decode(response[1]!.toString()));
      }
      return new PurchasesResponse(purchases, error);
    }
    return new PurchasesResponse(null, null);
  }

  static Future<MakePurchaseResponse> makePurchase(Product product) async {
    List<Object?>? response = await _channel.invokeMethod('makePurchase', {"product": json.encode(product.toJson())});
    MakePurchaseResponse? result = new MakePurchaseResponse(null, null);
    if (response != null) {
      if (response[0] != null) {
        var jsonResponse = json.decode(response[0]!.toString());
        if (jsonResponse["purchaseInfo"] is List) {
          result = MakePurchaseResponse.fromJson(jsonResponse["purchaseInfo"][0]);
        } else {
          result = MakePurchaseResponse.fromJson(jsonResponse);
        }
      }
      if (response[1] != null) {
        GenericError error = GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new MakePurchaseResponse(null, error);
      }
      return result;
    }
    return result;
  }

  static Future<BaseResponse> consumePurchase(String purchaseToken) async {
    List<Object?>? response = await _channel.invokeMethod('consumePurchase', {"purchaseToken": purchaseToken});
    BaseResponse? result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error = GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  static Future<BaseResponse> consumePurchases(List<String> purchaseTokens) async {
    List<Object?>? response = await _channel.invokeMethod('consumePurchases', {"purchaseTokens": purchaseTokens});
    BaseResponse? result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error = GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  static Future<CreateUserRelationResponse> createUserIdRelation(String masterUserId) async {
    List<Object?>? response = await _channel.invokeMethod('createUserIdRelation', {"masterUserId": masterUserId});
    CreateUserRelationResponse result = new CreateUserRelationResponse(null, null);
    if (response != null) {
      if (response[0] != null) {
        UserRelation userRelation = UserRelation.fromJson(json.decode(response[0].toString()));
        result = new CreateUserRelationResponse(userRelation, null);
      }
      if (response[1] != null) {
        GenericError error = GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new CreateUserRelationResponse(null, error);
      }
      return result;
    }
    return result;
  }

  static Future<BaseResponse> deleteMasterUserIdRelation(String masterUserId) async {
    List<Object?>? response = await _channel.invokeMethod('deleteMasterUserIdRelation', {"masterUserId": masterUserId});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error = GenericError.fromJson(json.decode(response[1]!.toString()));
        result = BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  static Future<BaseResponse> deleteSubUserIdRelation(String subUserId) async {
    List<Object?>? response = await _channel.invokeMethod('deleteSubUserIdRelation', {"subUserId": subUserId});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error = GenericError.fromJson(json.decode(response[1]!.toString()));
        result = BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  static Future<BaseResponse> unsubscribe(String purchaseToken) async {
    List<Object?>? response = await _channel.invokeMethod('unsubscribe', {"purchaseToken": purchaseToken});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error = GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

}

