import 'dart:io';

class BaseResponse {
  final bool success;
  final GenericError? error;

  BaseResponse(this.success, this.error);
}

class ProductsResponse {
  final List<Product>? products;
  final GenericError? error;

  ProductsResponse(this.products, this.error);
}

class Product {
  final String? productId;
  final String? uspAppId;
  final String? defaultLanguage;
  final List<ProductLocale>? productLocales;
  final String? purchaseType;
  final String? status;
  final PriceInfo? defaultPrice;
  final List<PriceInfo>? prices;
  final List<String>? entitlements;
  final String? subsPeriod;
  final String? subGroupId;
  final String? subGroupName;
  final String? promotionId;
  final String? trialPeriod;
  final String? introductoryPeriod;
  final PriceInfo? disDefaultPrice;
  final List<PriceInfo>? disPrices;
  final String? gracePeriod;
  final String? appleSyncStatus;
  final SyncInfo? appleSyncInfo;
  final String? huaweiPromotionPriority;
  final String? googleSyncStatus;
  final SyncInfo? googleSyncInfo;
  final String? huaweiSyncStatus;
  final SyncInfo? huaweiSyncInfo;
  final List<PriceTier>? applePriceDetails;
  final PriceTier? appleActivePriceDetail;

  Product(this.productId, this.uspAppId, this.defaultLanguage, this.productLocales, this.purchaseType, this.status, this.defaultPrice, this.prices, this.entitlements, this.subsPeriod, this.subGroupId, this.subGroupName, this.promotionId, this.trialPeriod, this.introductoryPeriod, this.disDefaultPrice, this.disPrices, this.gracePeriod, this.appleSyncStatus, this.appleSyncInfo, this.huaweiPromotionPriority, this.googleSyncStatus, this.googleSyncInfo, this.huaweiSyncStatus, this.huaweiSyncInfo, this.applePriceDetails, this.appleActivePriceDetail);

  Product.fromJson(Map<String, dynamic> json)
      : productId = json['productId'],
        uspAppId = json['uspAppId'],
        defaultLanguage = json['defaultLanguage'],
        productLocales = json['productLocales'] != null ? List<ProductLocale>.from(json['productLocales'].map((model) => ProductLocale.fromJson(model))) : null,
        purchaseType = json['purchaseType'],
        status = json['status'],
        defaultPrice = json['defaultPrice'] != null ? PriceInfo.fromJson(json['defaultPrice']) : null,
        prices = json['prices'] != null ? List<PriceInfo>.from(json['prices'].map((model) => PriceInfo.fromJson(model))) : null,
        entitlements = json['entitlements'] != null ? List.from(json['entitlements']) : null,
        subsPeriod = json['subsPeriod'],
        subGroupId = json['subGroupId'],
        subGroupName = json['subGroupName'],
        promotionId = json['promotionId'],
        trialPeriod = json['trialPeriod'],
        introductoryPeriod = json['introductoryPeriod'],
        disDefaultPrice = json['disDefaultPrice'] != null ? PriceInfo.fromJson(json['disDefaultPrice']) : null,
        disPrices = json['disPrices'] != null ? List<PriceInfo>.from(json['disPrices'].map((model) => PriceInfo.fromJson(model))) : null,
        gracePeriod = json['gracePeriod'],
        appleSyncStatus = json['appleSyncStatus'],
        appleSyncInfo = json['appleSyncInfo'] != null ? SyncInfo.fromJson(json['appleSyncInfo']) : null,
        huaweiPromotionPriority = json['huaweiPromotionPriority'],
        googleSyncStatus = json['googleSyncStatus'],
        googleSyncInfo = json['googleSyncInfo'] != null ? SyncInfo.fromJson(json['googleSyncInfo']) : null,
        huaweiSyncStatus = json['huaweiSyncStatus'],
        huaweiSyncInfo = json['huaweiSyncInfo'] != null ? SyncInfo.fromJson(json['huaweiSyncInfo']) : null,
        applePriceDetails = json['applePriceDetails'] != null ? List<PriceTier>.from(json['applePriceDetails'].map((model) => PriceTier.fromJson(model))) : null,
        appleActivePriceDetail = json['appleActivePriceDetail'] != null ? PriceTier.fromJson(json['appleActivePriceDetail']) : null;

    Map<String, dynamic> toJson() => {
      'productId': productId,
      'uspAppId': uspAppId,
      'defaultLanguage': defaultLanguage,
      'productLocales': productLocales != null ? List<dynamic>.from(productLocales!.map((model) => model.toJson())) : null,
      'purchaseType': purchaseType,
      'status': status,
      'defaultPrice': defaultPrice != null ? defaultPrice!.toJson() : null,
      'prices': prices != null ? List<dynamic>.from(prices!.map((model) => model.toJson())) : null,
      'entitlements': entitlements,
      'subsPeriod': subsPeriod,
      'subGroupId': subGroupId,
      'subGroupName': subGroupName,
      'promotionId': promotionId,
      'trialPeriod': trialPeriod,
      'introductoryPeriod': introductoryPeriod,
      'disDefaultPrice': disDefaultPrice != null ? disDefaultPrice!.toJson() : null,
      'disPrices': disPrices != null ? List<dynamic>.from(disPrices!.map((model) => model.toJson())) : null,
      'gracePeriod': gracePeriod,
      'appleSyncStatus': appleSyncStatus,
      'appleSyncInfo': appleSyncInfo != null ? appleSyncInfo!.toJson() : null,
      'huaweiPromotionPriority': huaweiPromotionPriority,
      'googleSyncStatus': googleSyncStatus,
      'googleSyncInfo': googleSyncInfo != null ? googleSyncInfo!.toJson() : null,
      'huaweiSyncStatus': huaweiSyncStatus,
      'huaweiSyncInfo': huaweiSyncInfo != null ? huaweiSyncInfo!.toJson() : null,
      'applePriceDetails': applePriceDetails != null ? List<dynamic>.from(applePriceDetails!.map((model) => model.toJson())) : null,
      'appleActivePriceDetail': appleActivePriceDetail != null ? appleActivePriceDetail!.toJson() : null,
    };

    String price() {
      if (Platform.isAndroid) {
        var price = defaultPrice?.price ?? "";
        if (price.length > 0) {
          return (double.parse(price) / 100).toStringAsFixed(2);
        }
      } else if (Platform.isIOS) {
        return appleActivePriceDetail?.price ?? "";
      }
      return "";
    }

    String currency() {
      if (Platform.isAndroid) {
        return defaultPrice?.currency ?? "";
      } else if (Platform.isIOS) {
        return appleActivePriceDetail?.currency ?? "";
      }
      return "";
    }
}

class ProductLocale {
  final String? language;
  final String? productName;
  final String? productDesc;

  ProductLocale(this.language, this.productName, this.productDesc);

  ProductLocale.fromJson(Map<String, dynamic> json)
      : language = json['language'],
        productName = json['productName'],
        productDesc = json['productDesc'];

  Map<String, dynamic> toJson() => {
    'language': language,
    'productName': productName,
    'productDesc': productDesc
  };
}

class PriceInfo {
  final String? country;
  final String? currency;
  final String? price;

  PriceInfo(this.country, this.currency, this.price);

  PriceInfo.fromJson(Map<String, dynamic> json)
      : country = json['country'],
        currency = json['currency'],
        price = json['price'];

  Map<String, dynamic> toJson() => {
    'country': country,
    'currency': currency,
    'price': price
  };
}

class PriceTier {
  final String? tier;
  final String? currency;
  final String? price;
  final String? startDate;
  final String? endDate;

  PriceTier(this.tier, this.currency, this.price, this.startDate, this.endDate);

  PriceTier.fromJson(Map<String, dynamic> json)
      : tier = json['tier'],
        currency = json['currency'],
        price = json['price'],
        startDate = json['startDate'],
        endDate = json['endDate'];

  Map<String, dynamic> toJson() => {
    'tier': tier,
    'currency': currency,
    'price': price,
    'startDate': startDate,
    'endDate': endDate
  };
}

class SyncInfo {
  final String? status;

  SyncInfo(this.status);

  SyncInfo.fromJson(Map<String, dynamic> json)
      : status = json['status'];

  Map<String, dynamic> toJson() => {
    'status': status
  };
}

class PurchasesResponse {
  final List<PurchaseInfo>? purchases;
  final GenericError? error;

  PurchasesResponse(this.purchases, this.error);
}

class MakePurchaseResponse {
  final PurchaseInfo? purchaseInfo;
  final GenericError? error;

  MakePurchaseResponse(this.purchaseInfo, this.error);

  MakePurchaseResponse.fromJson(Map<String, dynamic> json)
      : purchaseInfo = json['purchaseInfo'] != null ? PurchaseInfo.fromJson(json['purchaseInfo']) : null,
        this.error = null;
}

class PurchaseInfo {
  final String? userId;
  final String? uspAppId;
  final String? productId;
  final String? orderId;
  final String? purchaseTime;
  final int? purchaseStatus;
  final String? purchaseToken;
  final bool? autoRenew;
  final String? developerPayload;
  final String? platform;
  final bool? sandbox;
  final String? purchaseType;
  final int? consumed;
  final String? country;
  final String? currency;
  final String? price;
  final String? subGroupId;
  final String? cancelReason;
  final String? expireTime;
  final String? lastOrderId;
  final bool? inTrialPeriod;
  final String? introductoryPrice;
  final String? renewPrice;
  final String? graceExpireTime;
  final int? renewTime;
  final int? discountRenewTime;
  final String? nextRenewTime;
  final SubscriptionPriceChange? priceChange;
  final String? autoResumeTime;
  final String? linkedPurchasedToken;
  final String? receiptData;

  PurchaseInfo(this.userId, this.uspAppId, this.productId, this.orderId, this.purchaseTime, this.purchaseStatus, this.purchaseToken, this.autoRenew, this.developerPayload, this.platform, this.sandbox, this.purchaseType, this.consumed, this.country, this.currency, this.price, this.subGroupId, this.cancelReason, this.expireTime, this.lastOrderId, this.inTrialPeriod, this.introductoryPrice, this.renewPrice, this.graceExpireTime, this.renewTime, this.discountRenewTime, this.nextRenewTime, this.priceChange, this.autoResumeTime, this.linkedPurchasedToken, this.receiptData);

  PurchaseInfo.fromJson(Map<String, dynamic> json)
      : userId = json['userId'],
        uspAppId = json['uspAppId'],
        productId = json['productId'],
        orderId = json['orderId'],
        purchaseTime = json['purchaseTime'],
        purchaseStatus = json['purchaseStatus'],
        purchaseToken = json['purchaseToken'],
        autoRenew = json['autoRenew'],
        developerPayload = json['developerPayload'],
        platform = json['platform'],
        sandbox = json['sandbox'],
        purchaseType = json['purchaseType'],
        consumed = json['consumed'],
        country = json['country'],
        currency = json['currency'],
        price = json['price'],
        subGroupId = json['subGroupId'],
        cancelReason = json['cancelReason'],
        expireTime = json['expireTime'],
        lastOrderId = json['lastOrderId'],
        inTrialPeriod = json['inTrialPeriod'],
        introductoryPrice = json['introductoryPrice'],
        renewPrice = json['renewPrice'],
        graceExpireTime = json['graceExpireTime'],
        renewTime = json['renewTime'],
        discountRenewTime = json['discountRenewTime'],
        nextRenewTime = json['nextRenewTime'],
        priceChange = json['priceChange'] != null ? SubscriptionPriceChange.fromJson(json['priceChange']) : null,
        autoResumeTime = json['autoResumeTime'],
        linkedPurchasedToken = json['linkedPurchasedToken'],
        receiptData = json['receiptData'];
}

class SubscriptionPriceChange {
  final String? priceChangeDesc;

  SubscriptionPriceChange(this.priceChangeDesc);

  SubscriptionPriceChange.fromJson(Map<String, dynamic> json)
      : priceChangeDesc = json['priceChangeDesc'];
}

class ResultInfo {
  final String? code;
  final String? msg;

  ResultInfo(this.code, this.msg);

  ResultInfo.fromJson(Map<String, dynamic> json)
      : code = json['code'],
        msg = json['msg'];
}

class CreateUserRelationResponse {
  final UserRelation? data;
  final GenericError? error;

  CreateUserRelationResponse(this.data, this.error);

  CreateUserRelationResponse.fromJson(Map<String, dynamic> json)
      : data = json['data'] != null ? UserRelation.fromJson(json['data']) : null,
        error = null;
}

class UserRelation {
  final String? masterUser;
  final String? slaveUser;

  UserRelation(this.masterUser, this.slaveUser);

  UserRelation.fromJson(Map<String, dynamic> json)
      : masterUser = json['masterUser'],
        slaveUser = json['slaveUser'];
}

class GenericError {
  final String? message;

  GenericError(this.message);

  GenericError.fromJson(Map<String, dynamic> json)
      : message = json['message'];
}